
.q-grid-container * {
  -webkit-box-sizing: content-box;
  -moz-box-sizing: content-box;
  box-sizing: content-box;
}

.q-grid-container {
  position: relative;
  overflow-x: auto;
  width: auto;
  height: auto;
  border-radius: 4px;
  color: #111111;
}

.q-grid-container .hidden {
  display: none !important;
}

.q-grid-container a {
  color: #337ab7 !important;
  text-decoration: none;
  line-height: 20px;
}

.q-grid-container a:hover,
.q-grid-container a:focus {
  color: #23527c;
  text-decoration: underline;
}

.q-grid {
  position: relative;
  border: 1px solid #d3d3d3;
  margin-top: 6px;
  margin-bottom: 2px;
  box-sizing: border-box;
}

.q-grid-toolbar {
  margin: 4px 0
}

.q-grid-toolbar .btn {
  background-color: #EEEEEE;
  border-color: #E0E0E0;
  margin-right: 6px;
  padding: 4px 40px;
  /* taken from bootstrap's btn class, since
     jupyterlab doesn't include bootstrap css */
  display: inline-block;
  margin-bottom: 0;
  font-weight: normal;
  text-align: center;
  vertical-align: middle;
  touch-action: manipulation;
  cursor: pointer;
  background-image: none;
  border: 1px solid #E0E0E0;
  white-space: nowrap;
  font-size: 13px;
  line-height: 1.42857143;
  border-radius: 2px;
}

.q-grid-toolbar .btn:hover {
  background-color: #DDDDDD;
  border-color: #CCCCCC;
}

.q-grid-toolbar .btn:focus {
  outline: none;
}

.q-grid-toolbar .close-modal-btn {
  display: none;
}

.qgrid-modal .q-grid-toolbar .close-modal-btn {
  display: block;
}

.qgrid-modal .full-screen-btn,
.qgrid-modal .modal-header,
.qgrid-modal .modal-footer {
  display: none;
}

.qgrid-modal {
  width: 100%;
  height: 100%;
  margin: 0px;
}

.qgrid-modal .modal-content {
  position: absolute;
  left: 0px;
  right: 0px;
  top: 0px;
  bottom: 0px;
}

.qgrid-modal .modal-body {
  position: absolute;
  padding: 15px;
  top: 0px;
  left: 0px;
  bottom: 0px;
  right: 0px;
}

.qgrid-modal .modal-body button.close {
  margin-top: 7px;
  margin-right: 14px;
}

.qgrid-modal .q-grid-container {
  bottom: 25px;
  left: 20px;
  right: 20px;
  position: absolute;
  top: 15px;
}

.qgrid-modal .q-grid {
  position: absolute;
  top: 33px;
  bottom: 0px;
  left: 0px;
  right: 0px;
  height: auto !important;
}

.q-grid-toolbar .full-screen-btn,
.q-grid-toolbar .close-modal-btn {
  height: 18px;
  width: 12px;
  padding: 4px 7px;
  float: right;
  margin-right: 0px;
}

.output_scroll .q-grid {
  height: 315px !important;
}

.q-grid-toolbar {
  display: none;
}

.show-toolbar .q-grid-toolbar {
  display: block;
}

.output_scroll .show-toolbar .q-grid {
  height: 284px !important;
}

.q-grid.force-fit-columns .slick-viewport {
  overflow-x: hidden !important;
}

.q-grid.hide-scrollbar > .slick-header {
  margin-bottom: -1px;
}

.q-grid.hide-scrollbar > .slick-viewport {
  overflow-y: hidden !important;
  height: 100%;
}

.q-grid.hide-scrollbar.force-fit-columns > .slick-viewport {
  overflow: hidden !important;
}

.q-grid.hide-scrollbar > .slick-viewport > .grid-canvas {
  margin-top: 1px;
}

.q-grid .slick-header {
  border-top: none;
  border-left: none;
  border-right: none;
}

.q-grid .slick-header-columns,
.text-filter-grid .slick-header-columns {
  background: none;
  background-color: rgb(245, 245, 245);
  border-bottom: none;
}

.q-grid.force-fit-columns .slick-header-column:last-child .filter-button:hover,
.q-grid.force-fit-columns .slick-header-column.active:last-child .filter-button,
.q-grid.force-fit-columns .slick-header-column:last-child .filter-button.filter-active {
  border-right: 1px solid #ccc;
  margin-right: -4px;
}

.q-grid.force-fit-columns .slick-header-column:last-child .filter-button {
  margin-right: -4px;
  border-right: 1px solid transparent;
}

.q-qgrid.hide-scrollbar .slick-header-column:last-child .slick-resizable-handle {
  right: -4px;
}

.q-grid.hide-scrollbar > .slick_header .slick-header-column:last-child .filter-button,
.q-grid.hide-scrollbar > .slick-header .slick-header-column:last-child .filter-button:hover,
.q-grid.hide-scrollbar > .slick-header .slick-header-column.active:last-child .filter-button:hover,
.q-grid.hide-scrollbar > .slick-header .slick-header-column.active:last-child .filter-button.filter-active {
  border-right: 1px solid transparent;
}

.q-grid .slick-header-column .filter-button.filter-active,
.q-grid .slick-header-column .filter-button.filter-active:hover{
  background-color: #deeaf7;
  border-left: 1px solid #cccccc;
}

.q-grid .slick-header-column {
  background: rgb(245, 245, 245);
  color: #565656;
  border-right-style: none;
  font-size: 13px;
  font-weight: bold;
  padding: 6px 2px !important;
}

.q-grid .slick-header-column.active {
  background-color: #deeaf7;
}

.q-grid .slick-header-column-sorted {
  font-style: inherit !important;
}

.q-grid .slick-header-column:first-child .slick-column-name {
  margin-left: 6px !important;
}

.q-grid .slick-sort-indicator {
  height: 8px !important;
  margin-left: 6px;
}

.q-grid .slick-sort-indicator.fa-sort-asc {
  margin-top: 4px;
  color: #5862ff;
}

.q-grid .slick-sort-indicator.fa-sort-desc {
  margin-top: 1px;
  color: #5862ff;
}

.q-grid .slick-sort-indicator.fa-spinner {
  height: auto !important;;
  width: auto !important;;
  margin-top: 3px;
}

.q-grid .fa-check {
  color: #17a700;
}

.q-grid .slick-column-name {
  margin-bottom: 0px;
  float: left;
}

.q-grid .slick-resizable-handle {
  width: 1px;
  background-color: #c5c5c5;
  border-left: 3px solid rgb(245, 245, 245);
  border-right: 3px solid rgb(245, 245, 245);
}

.q-grid .slick-resizable-handle.active {
  border-right-color: #deeaf7;
}

.q-grid .slick-cell {
  border-bottom: 1px solid #e1e8ed;
  border-right: none;
  border-left: 1px solid transparent;
  font-size: 13px;
  font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
  padding-left: 0px;
  padding-top: 4px;
  border-top: none;
}

.q-grid.highlight-selected-row .slick-cell.selected {
  background-color: #deeaf7;
}

.q-grid.highlight-selected-cell .slick-cell.active {
  border: 2px solid rgb(65, 165, 245);
  padding-top: 2px;
  padding-bottom: 1px;
  padding-left: 3px;
}

.q-grid .slick-cell.editable:not(.idx-col) {
  border: 2px solid rgb(65, 165, 245);
  padding-top: 2px;
  padding-bottom: 1px;
  padding-left: 3px !important;
  background-color: #FFF;
  -webkit-box-shadow: 0 2px 5px rgba(0,0,0,0.4);
  -moz-box-shadow: 0 2px 5px rgba(0,0,0,0.4);
  box-shadow: 0 2px 5px rgba(0,0,0,0.4);
}

.q-grid .slick-cell.editable .editor-select:focus {
  outline-style: none;
}

.q-grid .slick-cell.idx-col {
  font-weight: bold;
  margin-right: 3px;
}

.q-grid .slick-cell.idx-col.last-idx-col {
  border-right: 1px solid rgb(225, 232, 237);
}

.q-grid .slick-cell.idx-col:not(.first-idx-col) {
  font-weight: bold;
  border-left: 1px solid rgb(225, 232, 237);
  margin-right: 3px;
}

.q-grid .idx-col.group-top {
  border-bottom-color: transparent;
  background-color: #FFF;
}

.q-grid .idx-col.group-middle {
  border-top-color: transparent;
  border-bottom-color: transparent;
  color: transparent;
  background-color: #FFF;
}

.q-grid .idx-col.group-bottom {
  border-top-color: transparent;
  color: transparent;
  background-color: #FFF;
}

.q-grid .idx-col.group-single {
  background-color: transparent;
}

.q-grid .slick-cell.l0.r0 {
  border-left: none;
  padding-left: 6px;
  z-index: 85;
}

.q-grid .slick-cell.l1.r1 {
  z-index: 86;
}

.q-grid .slick-cell.l2.r2 {
  z-index: 87;
}

.q-grid .slick-cell.l3.r3 {
  z-index: 88;
}

.q-grid .slick-cell.l4.r4 {
  z-index: 89;
}

.q-grid .slick-cell.l5.r5 {
  z-index: 90;
}

.q-grid .slick-cell.editable {
  z-index: 91 !important;
}

.q-grid .slick-cell.selected {
  background-color: transparent;
}

/* Filter button */

.q-grid .slick-header-column.active .filter-button, .filter-button:hover {
  background-color: #deeaf7;
  border-left: 1px solid #cccccc;
}

.q-grid .slick-header-column.filter-button-disabled .filter-button {
  color: #bababa;
}

.q-grid .filter-button {
  position: absolute;
  right: 4px;
  top: 0px;
  line-height: 29px;
  height: 28px;
  width: 21px;
  text-align: center;
  z-index: 10;
  background-image: none;
  margin-top: 0px;
}

.q-grid .filter-button:hover {
  background-color: #efefef;
  border-left: 1px solid #cccccc;
}

/* Filter dropdowns */

.q-grid-container .grid-filter {
  z-index: 1000;
  position: absolute;
  padding: 0px;
  margin: 0px;
  max-height: 400px;
  display: block;
  border-radius: 0px;
  /* we previously got these style from bootstrap */
  float: left;
  min-width: 160px;
  list-style: none;
  font-size: 13px;
  text-align: left;
  background-color: #fff;
  border: 1px solid rgba(0, 0, 0, 0.15);
  box-shadow: 0 6px 12px rgba(0, 0, 0, 0.175);
  background-clip: padding-box;
}

.q-grid-container .filter-error-msg {
  padding: 8px 14px;
  width: 160px;
}

.q-grid-container .grid-filter h3.qgrid-popover-title {
  padding: 5px 30px 5px 11px;
  border-bottom: 1px solid #d3d3d3;
  border-radius: 0px;
  /* we previously got these style from bootstrap */
  margin: 0;
  font-size: 13px;
  background-color: #f7f7f7;
  font-family: inherit;
  font-weight: 500;
  line-height: 1.1;
  color: inherit;
  display: block;
}

.q-grid-container .grid-filter .dropdown-title {
  font-size: 13px;
}

.q-grid-container .grid-filter i.close-button {
  position: absolute;
  right: 11px;
  top: 7px;
  padding: 0;
  color: #ababab;
  cursor: pointer;
  background-image: none;
  margin-top: 0px;
}

.q-grid-container .grid-filter i.close-button:hover {
  color: #777777;
}

.q-grid-container .grid-filter .dropdown-footer {
  background-color: #f7f7f7;
  border-top: 1px solid #d3d3d3;
  padding: 4px 12px;
  height: 19px;
}

.q-grid-container .grid-filter .dropdown-footer a.reset-link {
  padding: 0px;
  float: right;
}

.q-grid-container .grid-filter .dropdown-footer a.select-all-link {
  padding: 0px;
}

.q-grid-container .date-range-filter .dropdown-body {
  margin: 15px 11px;
  width: 210px;
}

.q-grid-container .date-range-filter .start-date {
  margin-right: 4px;
  width: 80px;
}

.q-grid-container .date-range-filter .end-date {
  margin-left: 4px;
  width: 80px;
}

.q-grid-container .numerical-filter .dropdown-body {
  margin: 20px 23px 10px 20px;
  width: 207px;
  text-align: center;
}

.q-grid-container .numerical-filter .slider-range {
  margin-bottom: 4px;
}


.q-grid-container .numerical-filter .ui-slider-handle {
  width: 18px;
  padding: 0;
}

.q-grid-container .numerical-filter .range-separator {
  font-size: 28px;
  font-weight: 300;
  padding: 0 7px;
  color: #646a76;
  position: relative;
  top: 4px;
}

.text-filter-grid * {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}

.text-filter .slick-viewport {
  overflow-y: auto !important;
  overflow-x: hidden !important;
}

.text-filter .input-area {
  padding: 12px;
}

.text-filter .no-results {
  margin-left: 12px;
  margin-bottom: 10px;
}

.text-filter .search-input {
  height: 30px;
  width: 100%;
  margin-bottom: 0px;
  box-sizing: border-box;
}

.text-filter .text-filter-grid {
  width: 300px;
  height: 250px;
  box-sizing: border-box;
  border-top: 1px solid #d3d3d3;
}

.text-filter .slick-header {
  border: none;
}

.text-filter .slick-header-columns {
  height: 0px;
}

.text-filter .slick-cell {
  line-height: 30px;
  border-left: none;
  border-right: none;
  background-color: #ffffff !important;
  border-bottom: 1px solid #e1e8ed !important;
  padding: 0;
  text-align: left;
  cursor: pointer;
}

.text-filter .slick-row.odd .slick-cell {
  background: #fafafa !important;
}

.text-filter .slick-row.active .slick-cell {
  background-color: #deeaf7 !important;
  border-top: 1px solid transparent !important;
}

.text-filter .check-box-cell {
  text-align: center;
  line-height: 28px;
}

.text-filter .check-box-cell input {
  margin: 0;
}

/* from custom-bootstrap *****************/

.q-grid-container input.datepicker {
  padding: 4px 4px 2px 4px;
  border: 1px solid lightgrey;
  font-size: 13px;
  line-height: 18px;
  font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
}

/* jquery-ui overrides */

.ui-widget, .ui-widget input, .ui-widget select, .ui-widget textarea, .ui-widget button {
  font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
  font-size: 13px;
}

.ui-tooltip {
  opacity: 1.0 !important;
}

/* jquery-ui datepicker style overrides */

#ui-datepicker-div {
  z-index: 9999 !important;
}

#ui-datepicker-div.ui-datepicker {
  font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
  /*@include box-shadow(0 5px 10px rgba(0, 0, 0, 0.2))*/
  z-index: 99 !important;
  padding: 0 !important;
  background-color: white;
  border: 1px solid gray;
}

#ui-datepicker-div.ui-datepicker .ui-icon {
  display: inline-block;
  font-family: "Font Awesome 5 Free", FontAwesome;
  font-size: inherit;
  text-rendering: auto;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  text-indent: 0px;
  margin-left: -4px;
  margin-top: -6px;
  background-image: none;
}

#ui-datepicker-div.ui-datepicker .ui-icon-circle-triangle-w:before {
    content: "\f053";
    font-weight: bold !important;
}

#ui-datepicker-div.ui-datepicker .ui-icon-circle-triangle-e:before {
    content: "\f054";
    font-weight: bold !important;
}


/* from slickgrid editing example */
input.editor-text {
  width: 100%;
  height: 100%;
  border: 0;
  margin: -1px 0 0 0;
  background: transparent;
  outline: 0;
  padding: 0;
}

.ui-datepicker-trigger {
  width: 16px;
  height: 16px;
  margin-top: 2px;
  padding: 0;
  vertical-align: top;
  left: 4px;
  position: relative;
}

.slick-cell.boolean {
  text-align: center;
}

.bool-radio-wrapper {
  width: 150px;
  position: relative;
  margin: 8px auto 4px auto;
}

.label-true {
  padding: 4px 28px 4px 13px;
}

.label-false {
  padding: 4px 28px 4px 4px;
  margin-left: 15px;
}

input.bool-filter-radio {
  margin-left: -23px;
}

.slick-cell {
  -webkit-touch-callout: none; /* iOS Safari */
    -webkit-user-select: none; /* Safari */
     -khtml-user-select: none; /* Konqueror HTML */
       -moz-user-select: none; /* Firefox */
        -ms-user-select: none; /* Internet Explorer/Edge */
            user-select: none;
}

.slick-row .slick-cell:not(:first-child) {
  padding-left: 5px;
  margin-left: -4px;
}

.q-grid .slick-sort-indicator-desc,
.q-grid .slick-sort-indicator-asc {
  background-image: none;
}

.jupyter-widgets {
  overflow: auto !important;
}
